<?php // $Id: mlm_ezmlm_idx.inc,v 1.5 2009/08/26 23:03:36 vauxia Exp $

/**
 * @file
 * EZMLM backend for the MLM module, supporting the IDX extensions for database
 * storage of subscribers.
 */

class mlm_ezmlm_idx extends mlm_ezmlm {

  var $autopost = NULL;
  var $autopost_key = NULL;

  function autopost() {
    return variable_get('mlm_ezmlm_idx_autopost', FALSE);
  }

  function edit_form(&$form) {
    $edit_form = parent::edit_form($form);

    $edit_form['troot'] = array(
      '#type'          => 'textfield',
      '#title'         => t('Table Prefix'),
      '#default_value' => $this->settings('troot'),
      '#description'   => t('Each list has a table prefix in the database to keep its data separate from other lists.  Enter the prefix that corresponds to the list\'s e-mail address.'),
    );
    $edit_form['key'] = array(
      '#type'          => 'textfield',
      '#title'         => t('List Key'),
      '#default_value' => $this->settings('key'),
      '#description'   => t('Required if using automatic posting.  Otherwise, you can safely ignore this'),
    );

    return $edit_form;
  }

  function edit_form_validate($form, &$form_state, &$values) {
    parent::edit_form_validate($form, $form_state, $values);

    if (!preg_match('/^[a-z0-9_]*$/i', $values['troot'])) {
      form_set_error('troot', t('Invalid format for table prefix'));
    }
    if ($this->autopost()) {
      if (!$values['key']) {
        form_set_error('key', t('List Key is required for auto-moderation'));
      }
    }
  }

  function edit_form_submit($form, &$form_state, &$values) {
    if ($this->autopost()) {
      $table   = $this->_table('post');
  
      $this->_db(TRUE);
      if (!db_table_exists($table)) {
        // We're not doing this in hook_schema because this table lives in the
        // ezmlm database.
        $table_schema = array(
          'fields' => array(
            'id' => array(
              'type' => 'serial',
            ),
            'website' => array(
              'type' => 'varchar',
              'length' => 100,
            ),
            'uid' => array(
              'type' => 'int',
              'unsigned' => TRUE,
              'not null' => TRUE,
              'default' => 0,
            ),
            'subject' => array(
              'type' => 'varchar',
              'length' => 200,
            ),
            'created' => array(
              'type' => 'int',
              'unsigned' => TRUE,
              'not null' => TRUE,
              'default' => 0,
            ),
            'sent' => array(
              'type' => 'int',
              'unsigned' => TRUE,
              'not null' => TRUE,
              'default' => 0,
            ),
          'primary key' => array('id'),
          ),
        );
        $ret = array();
        db_create_table($ret, $table, $table_schema);
      }
      $this->_db();
    }
  }

  function settings_form($node_type = NULL) {
    $form['mlm_ezmlm_idx_dsn'] = array(
      '#type'          => 'textfield',
      '#title'         => t('Connection Name'),
      '#default_value' => variable_get('mlm_ezmlm_idx_dsn', ''),
      '#description'   => t('If this list is stored in a separate database, <a href="http://drupal.org/node/18429" target="_blank">add the connection string to your settings file</a> and specify the connection name here.'),
    );
    $form['mlm_ezmlm_idx_autopost'] = array(
      '#type'          => 'checkbox',
      '#title'         => t('Enable Automatic Posting'),
      '#default_value' => variable_get('mlm_ezmlm_idx_autopost', 0),
      '#description'   => t('Allow posts to go through automatically, bypassing moderation and list controls. This is not a standard ezmlm feature and requires additional configuration.'),
    );

    return $form;
  }

  function settings_form_validate($form, $form_values) {
    parent::settings_form_validate($form, $form_values);
  }

  function subscribe() {
    $table =  $this->_table();
    $mail  = $this->account->mail;
    $this->_db(TRUE);

    db_query("INSERT INTO %s ( hash, address )
  		VALUES ( '%s', '%s' )", $table, ezmlm_hash($mail), $mail);

    db_query("INSERT INTO %s ( tai, address, fromline, edir) VALUES ( '%s', '%s', '%s', '+')"
      , $this->_table('slog'), date('Y-m-d H:i:s', time()), $mail, $mail);
    $this->_db();
    return TRUE;
  }

  function unsubscribe() {
    $mail = $this->account->mail;
    $table = $this->_table();
    $this->_db(TRUE);

    db_query("DELETE FROM %s WHERE address = '%s'", $table, $mail);
    if (db_affected_rows()) {
      db_query("INSERT INTO %s ( tai, address, fromline, edir) VALUES ( '%s', '%s', '%s', '-')"
        , $this->_table('slog'), date('Y-m-d H:i:s', time()), $mail, $mail);
    }
    $this->_db();
    return TRUE;
  }

  function count() {
    $table = $this->_table();
    $this->_db(TRUE);
    $count = db_result(db_query("SELECT COUNT(*) FROM $table"));
    $this->_db();
    return $count;
  }

  function subscriber_fields() {
    return array('mail' => t('E-mail'));
  }

  function subscribers($limit = NULL) {
    $table = $this->_table();
    $this->_db(TRUE);
    // it would be cool to join against the users table, but it may be a different DSN
    $func = $limit ? 'pager_query' : 'db_query';
    $res = $func("SELECT l.address AS mail FROM $table l
  		  ORDER BY l.address", $limit);
    $this->_db();
    return $res;
  }

  function is_subscribed() {
    $table = $this->_table();
    $this->_db(TRUE);
    $res = (boolean) db_result(db_query("SELECT 1 FROM %s
  		WHERE address = '%s' LIMIT 1", $table, $this->account->mail));
    $this->_db();
    return $res;
  }

  function post($message, $account = NULL) {
    $address = $this->settings('address');
    if ($this->autopost()) {
      global $base_url;
      global $user;
      $table   = $this->_table('post');
      $key     = $this->settings('key');
      $created = db_result(db_query("SELECT UNIX_TIMESTAMP()"));
      $hash    = sha1($base_url . $created . $key);
      $address = str_replace('@', "-autopost-{$hash}@", $address);

      $this->_db(TRUE);
      db_query("INSERT INTO %s (uid, website, subject, created, sent)
        VALUES (%d, '%s', '%s', %d, 0)",
        $table, $user->uid, $base_url, $message['subject'], $created);
      $this->_db();
    }
    mimemail($account, $address, $message['subject'], $message['body']);
  }

  function _table($type = 'subscriber') {
    $troot = $this->settings('troot');
    switch ( $type ) {
  
      case 'subscriber':
        return $troot;
  
      case 'moderator':
        return $troot .'_mod';
  
      case 'allow':
        return $troot .'_allow';
  
      case 'deny':
        return $troot .'_deny';
  
      case 'slog':
        return $troot .'_slog';
  
      case 'post':
        return $troot .'_post';
  
      default:
        return $troot;
        return FALSE;
    }
  }
  function _db($active = FALSE) {
    $db = $active ? variable_get('mlm_ezmlm_idx_dsn', NULL) : NULL;
    db_set_active($db);
  }
}
