<?php // $Id: admin.inc,v 1.14 2009/08/23 23:06:28 vauxia Exp $

/**
 * @file
 *
 * Helper and management functions for the MLM module.
 */

/**
 * Administration form for list management.
 */
function mlm_admin_list_overview() {
  $header = array(
    array('data' => t('Name'), 'field' => 'title', 'sort' => 'asc'),
    array('data' => t('Backend'), 'field' => 'backend'),
    array('data' => t('Type'), 'field' => 'type'),
    array('data' => t('Subscribers')),
    array('data' => t('Operations')),
  );
  $rows = array();

  $info = mlm_backends();
  $types = mlm_list_types();

  foreach (mlm_lists(NULL, NULL, NULL, $header, 50) as $mlm) {
    $rows[] = array(
      l($mlm->title(), $mlm->path()),
      $info[$mlm->backend]['title'],
      $types[$mlm->list_type]['title'],
      array('data' => l($mlm->count(), 'mlm/'. $mlm->lid .'/subscribers'), 'align' => 'right'),
      l(t('subscribers'), $mlm->path() .'/subscribers', array('query' => 'destination='. $_GET['q'])) .' '.
      l(t('edit'), $mlm->path() .'/edit', array('query' => 'destination='. $_GET['q'])),
    );
  }

  if (!$rows) {
    $rows[] = array(array('data' => t('No lists available.'), 'colspan' => '4'));
  }

  $output = theme('table', $header, $rows);
  $output .= theme('pager', NULL, 50, 0);
  return $output;
}

/**
 * Administration form for overall MLM features and settings.
 */
function mlm_admin_settings() {
  $form = array();
  $form['#validate'][] = '_mlm_admin_settings_validate';
  $form['#submit'][] = '_mlm_admin_settings_submit';

  // And a list of backends.
  $backends = mlm_backends();

  $form['general'] = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#title' => t('General settings'),
  );

  $form['general']['mlm_backends'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Enabled backends'),
    '#options' => $backend_list,
  );

  $enabled = array_filter(variable_get('mlm_backends', array()));

  foreach ($backends as $name => $info) {
    $form['general']['mlm_backends'][$name] = array(
      '#type' => 'checkbox',
      '#title' => $info['title'],
      '#description' => $info['description'],
      '#default_value' => isset($enabled[$name]),
    );

    $mlm = mlm_load($name);
    if ($settings = $mlm->settings_form()) {
      $id = 'edit-mlm-backends-'. str_replace('_', '-', $name);
      $form['settings'][$name] = array_merge($settings, array(
        '#type' => 'fieldset',
        '#title' => t('@name settings', array('@name' => $info['title'])),
        '#attributes' => array('class' => $id),
      ));
    }
  }

  $form['general']['mlm_help'] = array(
    '#type' => 'textarea',
    '#title' => t('Subscription help text'),
    '#description' => t('This text will appear on the !mlm subscriptions page.  Enter some helpful or descriptive text for users changing their preferences.', array('!mlm' => l('mlm', 'mlm'))),
    '#default_value' => variable_get('mlm_help', ''),
  );
  $form['general']['mlm_help_format'] = filter_form(variable_get('mlm_help_format', FILTER_FORMAT_DEFAULT), NULL, array('mlm_help_format'));

  foreach ($backends as $name => $info) {
  }

  // Hide the non-pertinent settings forms.
  $js = 'Drupal.behaviors.mlmSettings = function(context) {
    $("input[type=checkbox]").change( function() {
      var settingsForm = $("." + $(this).attr("id"));
      if ($(this).attr("checked") == 1) settingsForm.show();
      else settingsForm.hide();
    });
    $("input[type=checkbox]").trigger("change");
  }';
  drupal_add_js($js, 'inline');

  return system_settings_form($form);
}

/**
 * Validate handler for settings form.
 */
function mlm_admin_settings_validate($form, &$form_state) {
  foreach ($form_state['values']['mlm_backends'] as $backend => $val) {
    $mlm = mlm_load(NULL, $backend);
    $mlm->settings_form_submit($form, $form_state, $form_state['mlm'][$backend]);
  }
}

/**
 * Get an array of available list types ( e.g. announcement, discussion )
 * optionally filtered by backend(s) in use.
 */
function _mlm_list_types($backends='') {
  if ($backends) {
    if (!is_array($backends)) $backends = array($backends);
    $filtered = array();
    foreach ($backends as $backend) {
      if (!is_object($backend)) {
        $all_backends = mlm_backends();
        $backend = $all_backends[$backend];
      }
      foreach ($backend->supported_types as $t) {
        $filtered[$t] = t($t);
      }
    }
    return $filtered;
  }

  static $types = array();
  if (!count($types)) {
    foreach (mlm_backends() as $backend) {
      $types = array_merge($types, $backend->supported_types);
    }
    sort($types);
    foreach ($temp as $type) {
      $types[$type] = t($type);
    }
  }
  return $types;
}
