<?php // $Id: mlm.inc,v 1.12 2009/08/26 23:03:36 vauxia Exp $

class mlm {
  // The supported list types ( announcement, discussion, etc. ).
  var $supported_types = array();

  // The list id for this mailing list.
  var $lid = 0;

  // The node id that is acting as this list's container.
  var $nid = 0;

  // The title of this list.
  var $title = NULL;

  // The mailing list backend.
  var $backend = NULL;

  // The mailing list type (e.g. discussion, announcement).
  var $list_type = '';

  // The incoming address for post submissions.
  var $incoming_address = '';
  
  // An array of settings/options.
  // var $settings = array();

  // An array of settings/options.
  // var $text = array();

  function __construct($values = NULL) {
    $current = (array) $this;
    if ($values) {
      foreach ($values as $name => $val) {
        if ($name == 'settings' && !empty($val) && is_scalar($val)) {
          $this->__construct(unserialize($val));
        }
        elseif (array_key_exists($name, $current) && !is_null($val)) {
          $func = 'set_'. $name;
          if (method_exists($this, $func)) {
            $this->$func($val);
          }
          else {
            $this->$name = $val;
          }
          unset($current[$name]);
        }
      }
    }
    // Set defaults if possible.
    foreach ($current as $name => $val) {
      $func = 'set_'. $name;
      if (method_exists($this, $func)) {
        $this->$func($this->$name);
      }
    }
  }

  function access($op = 'view') {
    // TODO this should be fleshed out a bit better.
    if ($op == 'view') {
      return user_access('subscribe to mailing lists');
    }
    else {
      return user_access('administer mailing lists');
    }
  }

  function text_options() {
    return array(
      'notify_subscribe' => array(
        'title' => t('Send a "Welcome" message'),
        'subject' => TRUE,
        'body' => TRUE,
        'format' => TRUE,
      ),
      /*
      'confirm_subscribe' => array(
        'subject' => TRUE,
        'body' => TRUE,
        'format' => TRUE,
      ),
      'confirm_unsubscribe' => array(
        'subject' => TRUE,
        'body' => TRUE,
        'format' => TRUE,
      ),*/
      'notify_unsubscribe' => array(
        'title' => t('Send a "Goodbye" message'),
        'subject' => TRUE,
        'body' => TRUE,
        'format' => TRUE,
      ),
      'header' => array(
        'title' => t('Add a header to outgoing messages'),
        'subject' => FALSE,
        'body' => TRUE,
        'format' => TRUE,
      ),
      'footer' => array(
        'title' => t('Add a footer to outgoing messages'),
        'subject' => FALSE,
        'body' => TRUE,
        'format' => TRUE,
      ),
    );
  }

  function set_backend() {
    if (!isset($this->backend)) $this->backend = get_class($this);
  }

  function backend() {
    $this->set_backend();
    return $this->backend;
  }

  function set_title($val = NULL) {
    $this->title = check_plain($val);
  }

  function title() {
    return check_plain($this->title);
  }

  function set_supported_types($types = array('announcement', 'discussion')) {
    if (!$this->supported_types) {
      $info = mlm_list_types();
      $this->supported_types = array();
      foreach ($types as $name) {
        $this->supported_types[$name] = $info[$name]['title'];
      }
    }
  }

  function supported_types($name = NULL) {
    $this->set_supported_types();
    if ($name) return $this->supported_types[$name];
    return $this->supported_types;
  }

  function set_list_type($val = NULL) {
    if ($this->supported_types($val)) {
      $this->list_type = $val;
    }
  }

  function list_type() {
    return check_plain($this->list_type);
  }

  function set_incoming_address($val = NULL) {
    if (valid_email_address($val)) {
      $this->incoming_address = filter_xss($val);
    }
  }

  function incoming_address() {
    return filter_xss($this->incoming_address);
  }

  function set_description($text = NULL) {
    $this->description = filter_xss($text);
  }

  function description() {
    if ($this->nid) {
      $node = node_load($this->nid);
      return $node->teaser;
    }
    if ($description = $this->settings('description')) {
      return check_markup($description, $this->settings('format'));
    }
  }

  function set_settings($settings = array()) {
    // Load settings from database and set them in "$this->settings"
    if (!$this->settings) {
      $res = db_query("SELECT * FROM {mlm_setting} WHERE lid = %d", $this->lid);
      while ($row = db_fetch_object($res)) {
        if (!$value = @unserialize($row->value)) $value = $row->value;
        $this->settings[$row->name] = $value;
      }
    }
    if ($settings) {
      foreach ($settings as $name => $val) {
        $this->settings[check_plain($name)] = filter_xss($value);
      }
    }
  }

  function settings($name = NULL) {
    $this->set_settings();
    if ($name) return $this->settings[$name];
    return $this->settings;
  }

  function text($name = NULL) {
    // Load the text value.
    if (!isset($this->text[$name])) {
      $res = db_query("SELECT mtid, lid, list_type, subject, body, format 
        FROM {mlm_text} 
        WHERE name = '%s'
        AND (lid = %d)
        OR (lid IS NULL AND (list_type = '%s' OR list_type IS NULL))
        ORDER BY (lid = %d) DESC, (list_type = '%s') DESC LIMIT 1",
        $name, $this->lid, $this->list_type, $this->lid, $this->list_type);
      $this->text[$name] = db_fetch_object($res);
    }
    return $this->text[$name];
  }

  function token_list($type = 'all') {
    if ($type == 'mlm' || $type = 'all') {
      $tokens['mlm']['title'] = t('List name');
      $tokens['mlm']['list_type'] = t('List type');
      $tokens['mlm']['description'] = t('List description');
      $tokens['mlm']['path'] = t('Subscription URL');
    }
  }

  function token_values($type, $object = NULL) {
    if ($type == 'mlm') {
      $tokens['title'] = $this->title();
      $tokens['list_type'] = $this->list_type();
      $tokens['description'] = $this->description();
      $tokens['path'] = $this->path();

      return $tokens;
    }
  }

  function path() {
    if ($this->nid) {
      return 'node/'. $this->nid;
    }
    return 'mlm/'. $this->lid;
  }

  function account($account = NULL) {
    if(is_string($account)) {
      // It's simply an email address: Try loading it.
      $mail = $account;
      if (!$account = user_load(array('mail' => $mail))) {
        // Use the anonymous user instead, but store the mail address.
        $account = user_load(0);
        $account->mail = $mail;
      }
    }
    elseif(is_numeric($account)) {
      // It's presumably a user id.
      $account = user_load($account);
    }
    if (!is_object($account)) {
      // We've got nothing.  Use the logged-in user.
      global $user;
      $account = $user;
    }
    return $this->account = $account;
  }

  function settings_form($node_type = NULL) {
    return $form;
  }

  function settings_form_validate(&$form, &$form_state, $values) {
  }

  function settings_form_submit(&$form, &$form_state, $values) {
  }

  function import_form($form_state) {
    $form = array('#mlm_list' => $this);

    if (user_access('upload files')) {
      $form['import_file'] = array(
        '#type' => 'file',
        '#title' => t('Import subscribers from a file'),
      );
      $form['#attributes'] = array('enctype' => 'multipart/form-data');
    }
    $form['import_text'] = array(
      '#type' => 'textarea',
      '#rows' => 20,
      '#title' => t('Add subscribers manually'),
    );
    if ($this->settings('notify_subscribe')) {
      $form['notify'] = array(
        '#type' => 'checkbox',
        '#title' => t('Send Welcome E-mail'),
      );
    }
    $form['submit'] = array('#type' => 'submit', '#value' => t('add subscribers'));
    return $form;
  }

  function import_form_submit($form, &$form_state) {
    $values = $form_state['values'];
    if ($text = trim(preg_replace('/<[^>]*>/', ' ', $values['import_text']))) {
      $text = preg_replace(array('/\s/ms', '/,+/'), ',', $text);
      foreach (explode(',', $text) as $mail) {
        if (valid_email_address($mail)) {
          $this->_subscribe($mail, array(), $values['notify']);
        }
        else {
          drupal_set_message(t('%mail is not a valid email address', array('%mail' => $mail)), 'error');
        }
      }
    }
    if ($file = file_save_upload('import_file')) {
      batch_set(array(
        'title' => t('Importing subscribers'),
        'operations' => array(
          array('mlm_batch_import', array($this->lid, $file, $values['notify'])),
        ),
        'finished' => 'mlm_batch_done',
        'file' => drupal_get_path('module', 'mlm') .'/include/mlm.batch.inc',
      ));
    }
  }

  function export_form($form_state) {
    $form = array();
    $fields = $this->subscriber_fields();
    $options = array();
    foreach ($fields as $field => $name) $options[$field] = $field;
    $form['fields'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Fields to Export'),
      '#options' => $fields,
      '#default_value' => $options,
    );
    $form['submit'] = array('#type' => 'submit', '#value' => t('export'));
    return $form;
  }

  function export_form_submit($values) {
    ob_end_clean();
    drupal_set_header('Content-Type: text/csv');
    drupal_set_header('Content-Disposition: attachment; filename='. urlencode($this->title) .'-'. date('Y-m-d') .'.csv');
    $res = $this->subscribers();

    $header = array();
    foreach ($values['fields'] as $field) {
      if ($field) $header[] = strtoupper($field);
    }
    echo join(',', $header) ."\n";

    while ($row = db_fetch_object($res)) {
      $return = array();
      foreach ($values['fields'] as $field => $status) {
        if ($status) $return[] = $row->$field;
      }
      echo join(',', $return) ."\n";
    }
    die;
  }

  function _edit_form(&$form_state) {
    $form = array();
    $form['lid'] = array('#type' => 'value', '#value' => $this->lid);
    $form['backend'] = array('#type' => 'value', '#value' => $this->backend());

    if ($node = $form['#node']) {
      $form['nid'] = array('#type' => 'value', '#value' => $node->nid);
    }

    $form['title'] = array(
      '#type' => 'textfield',
      '#title' => t('List name'),
      '#default_value' => $this->title,
      '#weight' => -10,
    );

    $types = $this->supported_types();
    $form['list_type'] = array(
      '#type' => 'select',
      '#title' => t('List type'),
      '#options' => $types,
      '#default_value' => $this->list_type ? $this->list_type : current($types) ,
      '#access' => count($types) > 1,
    );

    $form['description']['description'] = array(
      '#type' => 'textarea',
      '#title' => t('Description'),
      '#default_value' => $this->settings('description'),
    );
    $form['description']['description_format'] = filter_form($this->settings('description_format'), NULL, array('description_format'));

    $form['incoming_address'] = array('#type' => 'value', '#value' => '');

    $form['text'] = array();
    foreach($this->text_options() as $name => $info) {
      $text = $this->text($name);
      $default = !$text->lid;
      $form['text'][$name] = array(
        '#type' => 'checkbox',
        '#title' => $info['title'],
        '#description' => $info['description'],
        '#default_value' => $this->settings($name),
      );
      $form['text'][$name. '_text'] = array( '#tree' => TRUE );
      if ($info['subject']) {
       $form['text'][$name .'_text']['subject'] = array(
          '#type' => 'textfield',
          '#title' => t('Subject'),
          '#default_value' => $default ? NULL : $text->subject,
        );
      }
      if ($info['body']) {
       $form['text'][$name .'_text']['body'] = array(
          '#type' => 'textarea',
          '#title' => t('body'),
          '#default_value' => $default ? NULL : $text->body,
        );
       $format = $default ? FILTER_FORMAT_DEFAULT : $text->format;
       if (is_null($format)) $format = FILTER_FORMAT_DEFAULT;
       $form['text'][$name .'_text'][$name .'_format'] = filter_form($format, NULL, array($name .'_text', $name .'_format'));
      }
    }
    if ($form['text']){
      $form['text']['#type'] = 'fieldset';
      $form['text']['#title'] = t('Instructions and help text');
      $form['text']['#collapsible'] = TRUE;
    }

    // Call the API's edit_form function to add additional fields.
    if (method_exists($this, 'edit_form')) {
      $form = array_merge($form, $this->edit_form($form));
    }

    return $form;
  }

  //function edit_form(&$form) {
  //}

  function _edit_form_validate(&$form, &$form_state, $values) {
    // Re-populate this class with new values, if any.
    $this->__construct($values);

    if (method_exists($this, 'edit_form_validate')) {
      $this->edit_form_validate($form, $form_state, $values);
    }
  }

  /**
   * API call to validate a mailing list form.
   *
   */
  //function edit_form_validate(&$form, &$form_state, $values) {
  //}

  function _edit_form_submit(&$form, &$form_state, &$values) {
    // Convert text settings to storable values.
    foreach($this->text_options() as $name => $info) {
      if ($values[$name]) {
        $text = $values[$name .'_text'];
        $this->text[$name] = array(
          'subject' => filter_xss($text['subject']),
          'body' => filter_xss($text['body']),
          'format' => filter_xss($text[$name .'_format']),
        );
      }
      // Remove it from the rest of the processing.
      unset ($values[$name .'_text']);
    }

    // Re-populate this class with new values, if any.
    $this->__construct($values);

    if (method_exists($this, 'edit_form_submit')) {
      $this->edit_form_submit($form, $form_state, $values);
    }
    // Store remaining values as settings, which will be stored during _save().
    foreach ($values as $name => $value) {
      if (!isset($this->$name)) $this->settings[$name] = $value;
    }
  }

  /**
   * API call to submit a mailing list form.
   *
   */
  //function edit_form_submit(&$form, &$form_state, $values = array()) {
  //}

  /**
   * API call to create a mailing list through the backend.
   *
   * @return
   *   Boolean status for the create operation.
   */
  //function save() {
  //}

  /**
   * The function called by Drupal to save a mailing list to maintain it
   * within our site.  You almost certainly don't want to override this: 
   * override save() instead.
   *
   * @return
   *   Boolean status for the storage operation.
   */
  function _save() {
    // Call the API's save function to do whatever actually saves the list.
    if (method_exists($this, 'save')) {
      $this->save();
    }

    // Save a new entry.
    if ($this->lid) {
      drupal_write_record('mlm', $this, 'lid');
      db_query("DELETE FROM {mlm_setting} WHERE lid = %d", $this->lid);
    }
    // Update an existing one.
    else {
      drupal_write_record('mlm', $this);
      $this->lid = db_last_insert_id('mlm', 'lid');
    }

    // Store text values for this list.
    foreach ($this->text_options() as $name => $info) {
      if ($this->settings[$name]) {
        $value = (object) $this->text[$name];
        $value->name = $name;
        $value->lid = $this->lid;
        $value->list_type = $this->list_type;

        $update = NULL;
        if ($mtid = db_result(db_query("SELECT mtid FROM {mlm_text} 
          WHERE lid = %d AND name = '%s'", $this->lid, $name))) {
          $value->mtid = $mtid;
          $update = 'mtid';
        }
        drupal_write_record('mlm_text', $value, $update);
      }
      else {
        db_query("DELETE FROM {mlm_text} 
          WHERE name = '%s' AND lid = %d", $name, $this->lid);
      }
    }

    // Store settings for this list.
    $filter = array('form_id', 'form_build_id', 'form_token', 'op', 'submit');
    foreach ($this->settings as $name => $value) {
      if (in_array($name, $filter)) continue;
      if (is_null($value) || !strlen($value)) continue;
      $row = array(
        'lid' => $this->lid,
        'type' => NULL,
        'name' => $name,
        'value' => is_scalar($value) ? $value : serialize($value),
      );
      drupal_write_record('mlm_setting', $row);
    }
  }

  function _delete() {
    // If the delete function has been overridden, call that too.
    if (method_exists($this, 'delete')) {
      $this->delete();
    }

    db_query("DELETE FROM {mlm} WHERE lid = %d", $this->lid);
    db_query("DELETE FROM {mlm_setting} WHERE lid = %d", $this->lid);
    db_query("DELETE FROM {mlm_slog} WHERE lid = %d", $this->lid);
    db_query("DELETE FROM {mlm_text} WHERE lid = %d", $this->lid);

    // Make sure this list isn't appearing in the subscription block.
    $block = variable_get('mlm_block_lists', array());
    unset($block[$this->mid]);
    variable_set('mlm_block_lists', $block);
  }

  function delete() {
  }

  function subscribe_form(&$form_state, $settings, $account = NULL, $lists = array()) {
    $account = $this->account($account);
    if (!$settings['show_mail'] && $account->uid) {
      $form['mlm_mail'] = array(
        '#type' => 'value',
        '#value' => $account->mail,
      );
    }
    else {
      $form['mlm_mail'] = array(
        '#type' => 'textfield',
        '#default_value' => $account->mail ? $account->mail : t('email address'),
        '#size' => 20,
        '#attributes' => array('onclick' => "this.value='';"),
        '#maxlength' => 80
      );
    }
    $form['subscribe_'. $this->lid] = array(
      '#type' => 'checkbox',
      '#title' => $this->title(),
      '#default_value' => $this->is_subscribed(),
      '#description' => $this->description(),
    );
    return $form;
  }

  function subscribe_validate($element, $form_state) {
    if (isset($element['#value']['mlm_mail'])) {
      if (!valid_email_address($element['#value']['mlm_mail'])) {
        form_set_error('mlm_mail', t('%mail is not a valid email address', array('%mail' => $form_state['values']['mlm_mail'])));
      }
    }
  }

  /**
   * API call to subscribe a user to a list.
   *
   * @param $accounts 
   *   The email addresses or user objects to subscribe.
   * @param $values
   *   Subscription settings, such as type or other values.
   * @param $notify
   *   Whether or not to send a welcome message.
   * @return
   *   Boolean result of the subscribe request.
   */
  function _subscribe($accounts = array(), $values = array(), $notify = TRUE) {
    if (!is_array($accounts)) $accounts = array($accounts);
    foreach ($accounts as $account) {
      $account = $this->account($account);

      // Don't re-subscribe.
      if (method_exists($this, 'is_subscribed')) {
        if ($this->is_subscribed()) continue;
      }

      // Handle the actual subscription request
      if (method_exists($this, 'subscribe')) {
        $this->subscribe($values);
      }
      
      // Send the user a notification, if desired and available.
      if ($notify) {
        $this->notify('subscribe', $account);
      }

      // Make a note of this subscription request.
      db_query("INSERT INTO {mlm_slog} ( lid, uid, mail, timestamp, status)
        VALUES ( %d, %d, '%s', %d, %d )",
        $this->lid, $account->uid, $account->mail, time(), 1);
    }
  }

  /**
   * API call to unsubscribe users from a list.
   *
   * @param $accounts
   *   An array of email addresses or user objects to subscribe.
   * @param $notify
   *   Whether or not to send a goodbye message.
   * @return
   *   Boolean result of the unsubscribe request.
   */
  function _unsubscribe($accounts = array(), $values = array(), $notify = TRUE) {
    if (!is_array($accounts)) $accounts = array($accounts);
    foreach ( $accounts as $account) {
      $account = $this->account($account);

      // Handle the actual subscription request
      if (method_exists($this, 'unsubscribe')) {
        $this->unsubscribe($values);
      }
      
      // Send the user a notification, if desired and available.
      if ($notify) {
        $this->notify('unsubscribe', $account);
      }

      // Make a note of this subscription request.
      db_query("INSERT INTO {mlm_slog} ( lid, uid, mail, timestamp, status)
        VALUES ( %d, %d, '%s', %d, %d )",
        $this->lid, $account->uid, $account->mail, time(), 0);
    }
  }

  function notify($op, $account) {
    if ($text = $this->text('notify_'. $op)) {
      mimemail(NULL, $account, $text->subject, $text->body);
    }
  }

  /**
   * API call to check whether a user is subscribed to a list.
   *
   * @param $account
   *   The email address or user object to check.
   * @return
   *   Boolean subscription status, or FALSE if unsupported (TODO).
   */
  function _is_subscribed($account = NULL) {
    $account = $this->account($account);
    if (method_exists($this, 'is_subscribed')) {
      return $this->is_subscribed();
    }
    return FALSE;
  }

  /**
   * API call to return a list of subscribers to a list.
   *
   * @param $filter
   *   An array of filters to the subscription list.
   * @return
   *   Database result of subscription query, or FALSE if unsupported.
   */
  function subscribers($filter = array()) {
  }

  /**
   * API call to return a list of fields that might be returned for a 
   * subscription list.
   *
   * @return
   *   Array of field names
   */
  function subscriber_fields() {
    return array();
  }

  /**
   * API call to return the count of all subscribers for a particular list.
   *
   * @param $filter
   *   An array of filters to the subscription list.
   * @return
   *  Numeric count of all subscribers, or FALSE if unsupported.
   */
  function count($filter = array()) {
  }

  /**
   * API call to post a message to a list.
   *
   * @param $account
   *   The user object that owns this post.
   * @param $subject
   *   The subject line for the post.
   * @param $body
   *   A post body (HTML or plaintext).
   * @param $msgid
   *   The message ID for the message that this post is responding to.
   * @return
   *   Boolean result of the post request.
   */
  function post($message, $account = NULL) {
    $account = $this->account($account);
  }

  /**
   * API call to archive a mailing list message.
   *
   * @param $message
   *   A mail message array, as parsed by mimemail_incoming.
   * @return
   *  The status of the archive command.
   */
  function archive($message = array()) {
    // Log traffic via send if it's available. TODO still in use?
    if ( function_exists('send_status') ) {
      send_status('mlm');
    }

    $headers = array(
      'Message-Id'    => $msgid,
      'Precedence'    => 'list',
      'X-Archived-By' => $_SERVER['SERVER_NAME'],
    );

    // TODO validate users' ability to post here
    if (variable_get('mimemail_incoming', FALSE)) {
      $headers['Reply-To'] = variable_get('mlm_drupal_incoming_address', '');
    }

    // TODO this is not robust or sane - use a better thread algo to find ref's
    $res = db_query("SELECT p.nid, p.msgid FROM {mlm_thread} p, {mlm_thread} m
      WHERE p.tid = m.pid AND m.msgid = '%s'", $msgid);
    if ($row = db_fetch_object($res)) {
      $headers['In-Reply-To'] = $row->msgid;
      $headers['References'] = $row->msgid;

      $res2 = db_query("SELECT msgid FROM {mlm_thread} WHERE cid = 0 AND nid = %d", $row->nid);
      if ($row2 = db_fetch_object($res2)) {
        $headers['References'] .= ' '. $row2->msgid;
      }
    }

    $subject = theme('mlm_post_subject', $list, $subject);
    $body    = theme('mlm_post_body', $list, $body);

    $this->post($user, $subject, $body);

    if ( $address = $this->post_address($list) ) {
      mimemail( $user, $address, $subject, $body, NULL, $headers );

      $count = $this->count();
      if ($count === NULL) {
        return count($address);
      }
      return $count;
    }
    return FALSE;
  }

  /**
   * API call to return the address used to post to a list.
   *
   * @return
   *   String containing the post addresss, or FALSE if unsupported.
   */
  function post_address() {
  }

  /*
   * Archive an incoming message received via hook_mimemail_incoming()
   */
  function receive($message) {
    global $user;
  
    // We'll be messing with the main user object for ownership and such.
    $old_user = $user;
  
    //  Make sure we haven't done this already.
    if (isset($message['headers']['X-Archived-By'])) {
      if ($message['headers']['X-Archived-By'] == $_SERVER['SERVER_NAME']) return;
    }
  
    //  Find parent message id for this thread.
    if (isset($message['headers']['In-Reply-To'])) {
      $parent_msgid = $message['headers']['In-Reply-To'];
    }
    elseif (isset($message['headers']['References'])) {
      $parent_msgid = $message['headers']['References'];
    }
    if ( $parent_msgid ) {
      $parent = db_fetch_object(db_query("SELECT * FROM {mlm_thread} WHERE msgid = '%s'", $parent_msgid));
    }
  
    // This message probably doesn't belong to MLM.
    if (!$parent) return;
  
    // TODO Validate user's list membership
    if ($new_user = user_load(array('mail' => $mail['from'])) ) {
      $user = $new_user;
    }
  
    $title     = $message['headers']['Subject'];
    $body      = theme('mlm_archive_text', $message['html']);
    $timestamp = strtotime($message['headers']['Date']);
  
    $status    = 1;
  
    if ( FALSE ) {  // TODO configurable node types.  For now we use comments.
      $type  = 'news';
      $node = array(
        'type' => $type,
        'mlm_parent' => $parent,
        'created'     => $timestamp,
        'name'        => $user->name,
        'mlm_message_source' => 'mail',
        'mlm_message_id' => $message['headers']['Message-Id'],
      );
      $values = array(
        'title'       => $title,
        'body'        => $body,
        'changed'     => time(),
        'status'      => $status,
        '#programmed' => TRUE,
      );
      drupal_execute($type .'_node_form', $values, $node);
    }
    else {
      $values = array(
        'timestamp'  => $timestamp,
        'uid'        => $user->uid,
        'status'     => 0, // TODO
        'name'       => $user->name,
        'mail'       => $message['from'],
        'subject'    => $title,
        'comment'    => $body,
        'format'     => FILTER_FORMAT_DEFAULT,
        'nid'        => $parent->nid,
        'pid'        => $parent->cid,
        'mlm_message_source' => 'mail',
        'mlm_message_id' => $message['headers']['Message-Id'],
      );
  
      // I couldn't get drupal_execute() to work for comments.
      $values['cid'] = comment_save($values);
  
      // comment_save() hard-codes the timestamp and makes assumptions
      // about status -- so we re-save
      comment_save($values);
    }
  
    $user = $old_user;
  }
  
  /*
   * Called from hook_nodeapi and hook_comment.
   * Determines the correct placement in the thread hierarchy and saves the value
   */
  function set_thread($node, $comment=NULL) {
    $cid    = $comment ? $comment['cid'] : 0;
    $uid    = $comment ? $comment['uid'] : $node->uid;
  
    // Already threaded
    if ($res = db_fetch_array(db_query("SELECT msgid FROM {mlm_thread} WHERE nid = %d AND cid = %d", $node->nid, $cid))) {
      return $res['msgid'];
    }
  
    $ts     = $comment ? $comment['timestamp'] : $node->created;
    // TODO - race conditions, but what else can ya do?
    $tid    = db_result(db_query("SELECT MAX(tid) FROM {mlm_thread}")) + 1;
  
    $source = 'drupal';  // Where this message came from ( e.g. drupal or mail )
    if ($comment && isset($comment['mlm_message_source'])) {
      $source = $comment['mlm_message_source'];
    }
    elseif (isset($node->mlm_message_source)) {
      $source = $node->mlm_message_source;
    }
  
    // Use the message id supplied by the user's MUA.
    $msgid  = '<'. $ts .'.'. $tid .'@'. $_SERVER['SERVER_NAME'] .'>';
    if ($comment && isset($comment['mlm_message_id'])) {
      $msgid  = $comment['mlm_message_id'];
    }
    elseif (isset($node->mlm_message_id)) {
      $msgid  = $node->mlm_message_id;
    }
  
    // Get the previous entry in the thread
    $pid = 0;
    if ($comment) {
      $row = db_fetch_object(db_query("SELECT tid FROM {mlm_thread} WHERE cid = %d", $comment['pid']));
      $pid = $row->tid;
    }
  
    db_query("INSERT INTO {mlm_thread} ( tid, pid, nid, cid, uid, source, msgid )
      VALUES ( %d, %d, %d, %d, %d, '%s', '%s' )", $tid, $pid, $node->nid, $cid, $uid, $source, $msgid );
  
    return $msgid;
  }
}
