<?php // $Id: mlm.subscription.inc,v 1.3 2009/08/26 23:03:36 vauxia Exp $

/**
 * A #process handler for mlm subscription elements.
 */
function mlm_subscription_element_process($element, $edit, &$form_state, $form){
  $settings = $element['#mlm_settings'];

  // Set the account we'll be processing.
  if (!$account = $element['#mlm_account']) {
    global $user;
    $account = $user;
  }

  if (isset($form_state['values'][$element['#name']])) {
    $element['#value'] = $form_state['values'][$element['#name']];
  }

  // Load and validate the lists referenced in this element.
  $lists = array();
  foreach ($element['#mlm_lists'] as $mlm) {
    if ($mlm = mlm_load($mlm)) {
      if ($settings['only_unsubscribed'] && $mlm->_is_subscribed($account)) {
        continue;
      }
      $lists[$mlm->lid] = $mlm;
    }
  }
  $element['#mlm_lists'] = $lists;

  $element['#tree'] = TRUE;

  // Call the mlm handler's subscription backend and merge its results.
  // this may allow them to share an email address, etc.
  $empty = TRUE;
  foreach ($lists as $mlm) {
    if ($subscribe_form = $mlm->subscribe_form($form_state, $settings, $account, $lists)) {
      $element = array_merge($element, $subscribe_form);
      $empty = FALSE;
    }

    // Hide a lonely checkbox if there's only one list to process.
    if ($settings['only_unsubscribed'] && count($lists) == 1) {
      $element['subscribe_'. $mlm->lid]['#type'] = 'value';
      $element['subscribe_'. $mlm->lid]['#value'] = TRUE;
    }
    elseif ($settings['single']) {
      $element['subscribe_'. $mlm->lid]['#type'] = 'value';
      $element['subscribe_'. $mlm->lid]['#value'] = !$mlm->_is_subscribed();
    }
  }

  // No lists to process. Skip out on adding extra text or form handling.
  if ($empty) {
    $element['#access'] = FALSE;
    return $element;
  }

  // Add some help text, if specified.
  if ($settings['helptext']) {
    $element['helptext'] = array(
      '#type' => 'markup',
      '#value' => $settings['helptext'],
      '#weight' => -5,
    );
  }

  // Include our validation callback.
  $element['#element_validate'] = array('mlm_subscription_validate');

  // Make sure that our submit handler is included in the form.
  if (!isset($form_state['submit_handlers'])) {
    $form_state['submit_handlers'] = array();
  }
  if (!in_array('mlm_subscription_submit', $form_state['submit_handlers'])) {
    $form_state['submit_handlers'][] = 'mlm_subscription_submit';
  }

  $element['#mlm_account'] = $account;

  return $element;
}

/**
 * A validate handler for the mlm_subscription element.
 */
function mlm_subscription_validate($element, &$form_state) {
  foreach ($element['#mlm_lists'] as $mlm) {
    $mlm = mlm_load($mlm);
    $mlm->subscribe_validate($element, $form_state);
  }
}

/**
 * A submit form for any form with an mlm_subscription element.
 */
function mlm_subscription_submit($form, &$form_state) {
  foreach (element_children($form) as $key) {
    if ($form[$key]['#type'] == 'mlm_subscription') {
      $settings = $form[$key]['#mlm_settings'];
      $account  = $form[$key]['#mlm_account'];
      $values = $form_state['values'][$key];

      foreach ($form[$key]['#mlm_lists'] as $mlm) {
        $mlm = mlm_load($mlm); 
        if ($values['subscribe_'. $mlm->lid]) {
          $mlm->_subscribe($account, $values);
        }
        else {
          $mlm->_unsubscribe($account, $values);
        }
      }
    }
  }
}

/**
 * Page callback to handle user subscription requests for all lists.
 */
function mlm_subscription_page($form_state, $account = NULL) {
  if (!$account) {
    global $user;
    $account = $user;
  }
  elseif (is_scalar($account) && valid_email_address($account)) {
    $mail = $account;
    $account = drupal_anonymous_user();
    $account->mail = $mail;
  }

  // Use an MLM form element to capture subscriptions.
  $form = array(
    'mlm' => array(
      '#type' => 'mlm_subscription',
      '#mlm_lists' => array(),
      '#mlm_settings' => array(),
    ),
  );

  $form['mlm']['#mlm_lists'] = mlm_lists(NULL, NULL, $mail);

  if (!count($form['mlm']['#mlm_lists'])) {
    drupal_set_message(t('There are no mailing lists available on this site', 'warning'));
  }
  else {
    $help = variable_get('mlm_help', '');
    $help = check_markup($help, variable_get('mlm_help_format', 0));

    if (user_access('administer mlm')) {
      $help .= l(t('Edit help text'), 'admin/build/mlm/settings');
    }

    $form['mlm']['#mlm_settings']['helptext'] = $help;
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save preferences'),
  );
  return $form;
}


/**
 * Menu callback for a "subscribe" and "unsubscribe" links on an MLM list.
 */
function mlm_subscription_callback(&$list, $op, $mail = NULL) {
  switch ($op) {
    case 'subscribe':
      $list->_subscribe($mail, TRUE);
      break;

    case 'unsubscribe':
      $list->_unsubscribe($mail, TRUE);
      break;
  }
  drupal_goto();
}
