<?php // $Id: mlm_node.admin.inc,v 1.1 2009/05/15 14:34:32 vauxia Exp $

function mlm_node_settings_form($type, $global = FALSE) {
  // Get a list of node types.
  $types = array();
  foreach (node_get_types() as $key => $type) {
    if ($key) $types[$key] = $type->name;
  }

  $form['general']['mlm_nodetypes'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Mailing list content types'),
    '#description' => t('Content types that can be used as mailing lists.  When nodes of these types are created, they will function as discussion or announcement lists, which users may subscribe to.'),
    '#options' => $types,
    '#default_value' => variable_get('mlm_nodetypes', array('mlm')),
    '#attributes' => array('id' => 'mlm-nodetypes'),
  );
  
  $form['types'] = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#title' => t('Content type settings'),
    '#attributes' => array('id' => 'mlm-nodetype-settings'),
  );
  
  foreach ($types as $type => $label) {
    $form['types'][$type] = _mlm_node_settings_form($type, TRUE);
    $form['types'][$type]['#type'] = 'fieldset';
    $form['types'][$type]['#title'] = $label;
    $form['types'][$type]['#attributes'] = array('id' => 'mlm-nodetype-'. $type);
  }
}

/**
 * MLM settings on a node type form or a node edit form.
 */
function _mlm_node_settings_form($type, $global = FALSE) {
  drupal_add_js(drupal_get_path('module', 'mlm') .'/js/mlm-node-edit.js');
  $form = array();

  $backends = _mlm_backends(NULL, !$global);
  $enabled_backends = _mlm_backends($type);

  // On the node type editing form, only show backends that are enabled.
  if (!$global) {
    $backends = $enabled_backends;
  }

  // This form is on both the general settings and the node type settings form,
  // which automatically prepends the type name to all variables.
  $prefix = $global ? $type .'_' : '';

  if (count($backends) == 1) {
    $form['mlm'][$prefix .'mlm_backends'] = array(
      '#type'         => 'value',
      '#value'        => array_keys($backends),
    );
  }
  else {
    // Select one or more backends for this node or node type.
    $form['mlm'][$prefix .'mlm_backends'] = array(
      '#type'          => 'checkboxes',
      '#title'         => t('Backends'),
    );
    foreach ($backends as $name => $mlm) {
      $form['mlm'][$prefix .'mlm_backends'][$name] = array(
        '#type'          => 'checkbox',
        '#title'         => $mlm->name,
        '#description'   => $mlm->description,
        '#return_value'  => $name,
        '#default_value' => isset($enabled_backends[$name]),
      );
    }
  }

  // list type selector
  if (count($types = _mlm_list_types($backends)) == 1) {
    $list_type = key($types);
  }
  if ($list_type) {
    $form['mlm'][$prefix .'mlm_list_type'] = array(
      '#type' => 'value',
      '#value' => $list_type,
    );
  }
  else {
    $form['mlm'][$prefix .'mlm_list_type'] = array(
      '#type'          => 'select',
      '#title'         => t('List Type'),
      '#options'       => $types,
      '#required'      => TRUE,
      '#default_value' => variable_get($prefix .'mlm_list_type', ''),
    );
  }

  $form['mlm']['messages'] = array(
    '#type'          => 'fieldset',
    '#title'         => t('Subscription messages'),
    '#collapsible'   => TRUE,
    '#collapsed'     => TRUE,
  );
  $form['mlm']['messages'][$prefix .'mlm_text_welcome'] = array(
    '#type'          => 'textarea',
    '#title'         => t('Welcome Text'),
    '#rows'          => 10,
    '#default_value' => variable_get($prefix .'mlm_text_welcome', ''),
    '#weight'        => 7,
  );
  $form['mlm']['messages'][$prefix .'mlm_text_goodbye'] = array(
    '#type'          => 'textarea',
    '#title'         => t('Goodbye Text'),
    '#rows'          => 10,
    '#default_value' => variable_get($prefix .'mlm_text_goodbye', ''),
    '#weight'        => 9,
  );
  if ( variable_get('mimemail_incoming', FALSE) ) {
    $form['mlm'][$prefix .'mail_handling'] = array(
      '#type'          => 'fieldset',
      '#title'         => t('Mail handling'),
      '#collapsible'   => TRUE,
      '#collapsed'     => TRUE,
    );
    $form['mlm']['mail_handling'][$prefix .'mlm_incoming_address'] = array(
      '#type'          => 'textfield',
      '#title'         => t('Incoming address'),
      '#description'   => t('The inbound address for posts and commands for this list.'),
      '#default_value' => variable_get($prefix .'mlm_incoming_address', ''),
    );
  }
  foreach (_mlm_backends() as $backend => $mlm) {
    if ($form['mlm'][$backend] = $mlm->settings_form($type)) {
      $form['mlm'][$backend]['#type'] = 'fieldset';
      $form['mlm'][$backend]['#attributes'] = array('id' => 'mlm-settings-'. $backend);
      $form['mlm'][$backend]['#title'] = t('%backend settings', array('%backend' => $mlm->name));
    }
  }
  return $form;
}

/**
 * Submit handler for node type settings form.
 */
function _mlm_admin_settings_submit($form, &$form_state) {
  foreach ($form_state['values']['mlm_backends'] as $backend => $val) {
    $mlm = mlm_load(NULL, $backend);
    $mlm->settings_form_submit($form, $form_state, $form_state['mlm'][$backend]);
  }

  $old = variable_get('mlm_nodetypes', array());
  $new = $form_state['values']['mlm_nodetypes'];

  // Remove and replace menu_link entries for the node edit forms.
  foreach ($old as $type => $val) {
    menu_link_maintain('mlm', 'delete', 'admin/user/mlm/form/'. $type, '');
  }
  foreach ($new as $type => $val) {
    if (!$val) continue;
    $menu_link = array(
      'link_title' => 'Add '. node_get_types('name', $type),
      'link_path' => 'admin/user/mlm/form/'. $type,
      'module' => 'mlm',
      'type' => MENU_LOCAL_TASK,
    );
    menu_link_save($menu_link);
  }
}
